import machine
import time
import neopixel



def set_row(rows, i):
    """
    Sets row to be read by index i
    :param rows: list of machine.Pin objects corresponding to signals row_sel from schematics
    :param i: index of row to be selected
    :return: nothing
    """
    rows[0].value(1&i)
    rows[1].value(2&i)
    rows[2].value(4&i)

def compute_R(V,Rg,Vr):
    """
    THIS FUNCTION IS TO BE IMPLEMENTED BY STUDENTS
    Computes resistance from measured voltage and known constants
    :param V: Measured voltage
    :param Rg: Gain resistor resistance
    :param Vr: Voltage reference voltage
    :return: Computed resistance
    """
    return -1

def measure_array(rows, adcs, shape):
    """
    THIS FUNCTION IS TO BE IMPLEMENTED BY STUDENTS
    Row-by-Row reads tactile array.
    :param rows: list of machine.Pin objects corresponding to signals row_sel from schematics
    :param adcs: list of machine.adc objects corresponding to AColx from schematics
    :param shape: number of rows and columns of tactile array
    :return: array of same shape as tactile array of measured resistances
    """
    result = [[0 for i in range(shape[0])] for j in range(shape[1])]
    
    return result

def send_array(data,shape):
    print("S",end=',')
    for i in range(shape[1]):
        for j in range(shape[0]):
            print(data[i][j],end=',')
    print()


adcs = []
adcs.append(machine.ADC(3))
adcs.append(machine.ADC(0))
adcs.append(machine.ADC(1))
adcs.append(machine.ADC(2))
adcs.reverse()
rows = []
rows.append(machine.Pin(11, machine.Pin.OUT))
rows.append(machine.Pin(10, machine.Pin.OUT))
rows.append(machine.Pin(9, machine.Pin.OUT))
set_row(rows,0)

# neopixel based test
# board LED goes from red to blue and main cycle starts
blue = [0,0,32] # G,R,B
red = [0,32,0] # G,R,B
ws_pin = 16
neo_led = neopixel.NeoPixel(machine.Pin(ws_pin), 1)
neo_led.fill(red)
neo_led.write()
time.sleep(1)
neo_led.fill(blue)
neo_led.write()

init_array = measure_array(rows,adcs,(4,4))
toWait = 50 #ms, 20 fps
while(42):
    start = time.ticks_ms()                             # start timestamp
    array = measure_array(rows,adcs,(4,4))
    send_array(array,(4,4))
    cycleTook = (time.ticks_ms()-start)                 # get exact time to wait
    if (cycleTook < toWait):
        time.sleep_ms(toWait-cycleTook)

